import React, { useEffect, useState } from 'react'
import { CloseOutlined, DeleteOutlined, PlusOutlined } from '@ant-design/icons';
import { TreeSelect, Form, Input, InputNumber, Radio, Modal, Switch } from 'antd'
import menuService from '@/request/service/menu';
import { useRequest } from '@/hooks/use-request';
import type { DefaultOptionType } from 'antd/es/select';
import type { Menu } from '@/models'
import { antdUtils } from '@/utils/antd';

const layout = {
    labelCol: { span: 4, },
    wrapperCol: { span: 16 },
    bordered: false,
};

interface MenuEditorProps {
    visible: boolean;
    onCancel: (flag?: boolean) => void;
    onSave: (menu: Menu) => void;
    editData?: Menu | null;
    parentMenus: Menu[];
}


const MenuEditor: React.FC<MenuEditorProps> = (props) => {

    const { visible, onCancel, onSave, editData, parentMenus } = props;

    const { runAsync: updateMenu } = useRequest(menuService.updateMenu, { manual: true });
    const { runAsync: createMenu } = useRequest(menuService.createMenu, { manual: true });

    const isEdit = !!editData;

    const menu2Tree = (menu: Menu) => {
        const treeItem: DefaultOptionType = {
            label: menu.name,
            value: menu.id
        }
        if (menu.children && menu.children.length > 0) {
            treeItem.children = menu.children.filter(item => item.type !== 3).map(menu2Tree)
        }

        return treeItem;
    }
    const menuTree: DefaultOptionType[] = [{
        label: '主类目',
        value: 0,
        children: parentMenus.filter(item => item.type !== 3).map(item => {
            return menu2Tree(item)
        })
    }]

    const [saveLoading, setSaveLoading] = useState(false);
    const [form] = Form.useForm();

    useEffect(() => {
        if (visible) {
            if (editData) {
                form.setFieldsValue(editData);
            }
        } else {
            form.resetFields();
        }
    }, [visible]);

    const save = async () => {
        setSaveLoading(true);
        const menuValues = form.getFieldsValue();
        if(typeof(menuValues.status) === 'boolean') {
            menuValues.status = menuValues.status? 0 : 1
        }
        const newValue = isEdit? {...editData, ...menuValues} : {
            permission: '',
            icon: '',
            keepAlive:true,
            createTime: new Date(),
            ...menuValues,
        }
        const [error, { msg, code }] = isEdit? await updateMenu(newValue): await createMenu(newValue);
        if(!error && code === 0) {
            onSave(newValue);
        } else {
            antdUtils.message?.open({
                type: 'error',
                content: msg??'操作失败',
            });
        }
        setSaveLoading(false);
    }

    return (
        <>
            <Modal
                open={visible}
                title={isEdit ? "编辑" : "新建"}
                width={640}
                onOk={save}
                onCancel={() => onCancel()}
                destroyOnClose
            >
                <Form
                    form={form}
                    {...layout}
                    onFinish={save}
                    labelCol={{ flex: '0 0 100px' }}
                    wrapperCol={{ span: 16 }}
                >
                    <Form.Item name="parentId" label="上级菜单" >
                        <TreeSelect treeData={menuTree} />
                    </Form.Item>

                    <Form.Item name="name" label="菜单名称"
                        rules={[
                            {
                                required: true,
                                message: '请输入菜单名称',
                            },
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="type" label="菜单类型" >
                        <Radio.Group options={[
                            { value: 1, label: "目录" },
                            { value: 2, label: "菜单" },
                            { value: 3, label: "按钮" }
                        ]} optionType="button">
                        </Radio.Group>
                    </Form.Item>

                    <Form.Item name="icon" label="菜单图标" >
                        <Input />
                    </Form.Item>

                    <Form.Item name="path" label="路由地址"
                        rules={[
                            {
                                required: true,
                                message: '请输入路由地址',
                            }
                        ]}
                    >
                        <Input />
                    </Form.Item>

                    <Form.Item name="component" label="组件地址" >
                        <Input />
                    </Form.Item>

                    <Form.Item name="sort" label="显示排序" >
                        <InputNumber min={1} max={100} />
                    </Form.Item>

                    <Form.Item name="status" label="菜单状态"
                        // getValueProps={(value: number) => {
                        //     const checked = value !== 0;
                        //     console.log(checked)
                        //     return { checked }
                        // }} 
                        valuePropName="checked"
                        // normalize={(value: boolean) => {
                        //     // prevValues.status = value ? 0 : 1
                        //     return value ? 0 : 1
                        // }}
                        >
                        <Switch checkedChildren="开启" unCheckedChildren="关闭" />
                    </Form.Item>

                    <Form.Item name="visible" label="显示状态" valuePropName="checked">
                        <Switch checkedChildren="显示" unCheckedChildren="隐藏" />
                    </Form.Item>

                    {/* <Form.Item
                        name="cache"
                        label="缓存状态"
                    >
                        <Switch />
                    </Form.Item> */}
                </Form>
            </Modal>
        </>
    )
}

export default MenuEditor;
